(* ::Package:: *)

(***********************************************************************)
(*** Mathematica script for dynamical Chern-Simons gravity 3+1 split ***)
(***********************************************************************)

Needs["xAct`xTensor`"];

$PrePrint = ScreenDollarIndices;
$ExtrinsicKSign = -1;
$AccelerationSign = -1;
Index = IndexRange[a, m];

(***********************************************************************)
(**********            Useful little collect rule           ************)
(***********************************************************************)
TermsWith[exp_, pat_] := Module[{i, pos, expr},
  expr = exp // Expand;
  pos = Position[expr, pat];
  Sum[expr[[pos[[i, 1]]]], {i, 1, Length[pos]}]
  ]

(***********************************************************************)
(**********                4-manifold                       ************)
(***********************************************************************)
DefManifold[M, 4, Index];

(***********************************************************************)
(**********               4-Metric                            **********)
(***********************************************************************)

DefMetric[-1, metric[-a, -b], CD, PrintAs -> "g"];

(***********************************************************************)
(**********               Dual Riemann and Weyl               **********)
(***********************************************************************)

RiemannStar[-a_, -b_, -c_, -d_] := Module[{e, f}, 
   1/2 epsilonmetric[e, f, -c, -d] RiemannCD[-e, -f, -a, -b]];
WeylStar[-a_, -b_, -c_, -d_] := Module[{e, f}, 
   1/2 epsilonmetric[e, f, -c, -d] WeylCD[-e, -f, -a, -b]];

(***********************************************************************)
(**********               Normal vector                       **********)
(***********************************************************************)
DefTensor[n[a], M];
n /: n[a_] n[-a_] := -1;
n /: n[a_] n[b_] metric[-a_, -b_] := -1;
n /: n[-a_] n[-b_] metric[a_, b_] := -1;
n /: n[a_] CD[b_][n[-a_]] := 0;
n /: n[a_] CD[-b_][n[-a_]] := 0;
n /: n[-a_] CD[b_][n[a_]] := 0;
n /: n[-a_] CD[-b_][n[a_]] := 0;

(***********************************************************************)
(**********                  Induced metric                   **********)
(***********************************************************************)
DefMetric[1, metrich[-a, -b], cd, 
  SymbolOfCovD -> {"|", "\[ScriptCapitalD]"}, 
  InducedFrom -> {metric, n}, PrintAs -> "h"];

RuleProj = Projectormetrich -> ProjectWith[metrich];
Proj = Function[X,X/.RuleProj];

(***********************************************************************)
(**********            Lapse, shift and time vector           **********)
(***********************************************************************)

DefTensor[lapse[], M, PrintAs -> "\[Alpha]"];
DefTensor[shift[a], M, OrthogonalTo -> n[-a], PrintAs -> "\[Beta]"];
DefTensor[t[a], M];

AccRule = MakeRule[{Accelerationn[-a],cd[-a][lapse[]]/lapse[]},MetricOn->All];
AccToLapse[e_] := e /. AccRule;


(***********************************************************************)
(**********            Kill Div R                            **********)
(***********************************************************************)

SpatialContractedBianchiRule=
MakeRule[{cd[-b][Riccicd[-a,b]],1/2cd[-a][RicciScalarcd[]]},MetricOn->All];

(***********************************************************************)
(**********            TF decomposition of K                  **********)
(***********************************************************************)

DefTensor[TrK[], M, PrintAs -> "K"];
DefTensor[AA[-a, -b], M, Symmetric[{1, 2}], OrthogonalTo -> n[a], 
  ProjectedWith -> {metrich[a, -c], metrich[b, -d]}, PrintAs -> "A"];
AA /: AA[a_, -a_] := 0
AA /: AA[-a_, a_] := 0


(* Non-trivial rule on the AAA products presumably altered in higher 
   dimensions *)
AAARule=MakeRule[{AA[-a,c]*AA[-b,d]*AA[-c,-d],
(AA[-a,-b]*AA[-c,-d]*AA[c,d])/2+AA[-d,c]*AA[d,e]*AA[-e,-c]*metrich[-a,-b]/3},MetricOn->All];

TraceFreeKRule = {
    MakeRule[{ExtrinsicKmetrich[-a, -b], 
      AA[-a, -b] + 1/3 metrich[-a, -b] TrK[]}],
    MakeRule[{ExtrinsicKmetrich[-a, b], 
      AA[-a, b] + 1/3 metrich[-a, b] TrK[]}],
    MakeRule[{ExtrinsicKmetrich[a, b], 
      AA[a, b] + 1/3 metrich[a, b] TrK[]}],
    MakeRule[{ExtrinsicKmetrich[-a, a], TrK[]}]
    } // Flatten;

TraceFreeK = Function[X, X/.TraceFreeKRule];

(***********************************************************************)
(**********            Decomposition of epsilon               **********)
(***********************************************************************)
epsilonmetric /: epsilonmetric[a_, b_, c_, d_] n[-a_] := 
  epsilonmetrich[b, c, d];
epsilonmetric /: epsilonmetric[-a_, -b_, -c_, -d_] n[a_] := 
  epsilonmetrich[-b, -c, -d];
DefTensor[eps[-a, -b, -c, -d], M, Antisymmetric[{-a, -b, -c, -d}]];
eps[-a_, -b_, -c_, -d_] = 
  Antisymmetrize[-4 n[-a] epsilonmetrich[-b, -c, -d], {-a, -b, -c, \
-d}] // Simplification;
ruleeps = {epsilonmetric[-a_, -b_, -c_, -d_] :> eps[-a, -b, -c, -d], 
   epsilonmetric[a_, -b_, -c_, -d_] :> 
    Module[{e}, eps[-e, -b, -c, -d] metric[a, e]],
   epsilonmetric[a_, b_, -c_, -d_] :> 
    Module[{e, f}, eps[-e, -f, -c, -d] metric[a, e] metric[b, f]],
   epsilonmetric[a_, b_, c_, -d_] :> 
    Module[{e, f, g}, 
     eps[-e, -f, -g, -d] metric[a, e] metric[b, f] metric[c, g]],
   epsilonmetric[a_, b_, c_, d_] :> 
    Module[{e, f, g, h}, 
     eps[-e, -f, -g, -h] metric[a, e] metric[b, f] metric[c, 
       g] metric[d, h]]};

DecomposeEps = Function[X,X/.ruleeps];

(***********************************************************************)
(**********           Electric and magnetic parts             **********)
(***********************************************************************)
DefTensor[EWeyl[a, b], M, Symmetric[{a, b}], 
 OrthogonalTo -> {n[-a], n[-b]}, 
	  ProjectedWith -> {metrich[-a, c], metrich[-b, d]}];
DefTensor[BWeyl[a, b], M, Symmetric[{a, b}], 
 OrthogonalTo -> {n[-a], n[-b]}, 
	  ProjectedWith -> {metrich[-a, c], metrich[-b, d]}];

EWeyl /: EWeyl[-a_, a_] := 0
BWeyl /: BWeyl[-a_, a_] := 0
EWeyl /: EWeyl[a_, -a_] := 0
BWeyl /: BWeyl[a_, -a_] := 0

PartToWeylRule = {EWeyl[a_, b_] :> 
    Module[{c, d}, n[-c] n[-d] WeylCD[a, c, b, d]],
   BWeyl[a_, c_] :> 
    Module[{b, d, e, f}, 
      n[-b] n[-d] epsilonmetric[c, d, -e, -f] WeylCD[a, b, e, f]]/2};

WeylToPartRule = 
  WeylCD[a_, b_, m_, l_] :> 
   Module[{c}, -(EWeyl[a, l]*metric[b, m]) + 
     EWeyl[a, m]*metric[b, l] - 
      BWeyl[b, c]*epsilonmetrich[m, l, -c]*n[a] + 
      BWeyl[a, c]*epsilonmetrich[m, l, -c]*n[b] - 
      BWeyl[l, c]*epsilonmetrich[a, b, -c]*n[m] - 
     2*EWeyl[a, l]*n[b]*n[m] + 
      EWeyl[b, l]*(metric[a, m] + 2*n[a]*n[m]) + 
      BWeyl[m, c]*epsilonmetrich[a, b, -c]*n[l] + 
     2*EWeyl[a, m]*n[b]*n[l] - 
      EWeyl[b, m]*(metric[a, l] + 2*n[a]*n[l])];

WeylToPart = Function[X, X /. WeylToPartRule];

DefTensor[X[-a, -b], M, Symmetric[{1, 2}], OrthogonalTo -> n[a], 
ProjectedWith -> {metrich[a, -c], metrich[b, -d]}];
X /: X[a_, -a_] := 0
X /: X[-a_, a_] := 0

(***********************************************************************)
(**********                TF Ricci Rules                     **********)
(***********************************************************************)
DefTensor[TrR[], M]
DefTensor[Rt[-a, -b], M, Symmetric[{1, 2}], OrthogonalTo -> n[a], 
  ProjectedWith -> {metrich[a, -c], metrich[b, -d]}];
Rt /: Rt[a_, -a_] := 0
Rt /: Rt[-a_, a_] := 0

TraceFreeRRule = {
    MakeRule[{Riccicd[-a, -b], 
      Rt[-a, -b] + 1/3 metrich[-a, -b] RicciScalarcd[]}],
    MakeRule[{Riccicd[-a, b], 
      Rt[-a, b] + 1/3 metrich[-a, b] RicciScalarcd[]}],
    MakeRule[{Riccicd[a, b], 
      Rt[a, b] + 1/3 metrich[a, b] RicciScalarcd[]}],
    MakeRule[{Riccicd[-a, a], RicciScalarcd[]}]
    } // Flatten;

TraceFreeRule = Union[TraceFreeKRule, TraceFreeRRule];

(***********************************************************************)
(**********    Rules for traces Lie, geometric relations      **********)
(***********************************************************************)
TraceLienAARule = 
  MakeRule[{metrich[a, b] LieD[n[c]][AA[-a, -b]], -2 AA[-a, -b] AA[a, 
      b]}];
TraceLienBWeylRule = 
  MakeRule[{metrich[a, b] LieD[n[c]][
      BWeyl[-a, -b]], -2 BWeyl[-a, -b] AA[a, b]}];
TraceLienEWeylRule = 
  MakeRule[{metrich[a, b] LieD[n[c]][
      EWeyl[-a, -b]], -2 EWeyl[-a, -b] AA[a, b]}];

TraceLienRules = 
  Union[TraceLienEWeylRule, TraceLienBWeylRule, TraceLienAARule];

(***********************************************************************)
(**********               E and B definitions                 **********)
(***********************************************************************)
Edefn[-e_, -f_] = 
 Module[{a, b, c, 
   d}, ((((ToInducedDerivative[
                    GaussCodazzi[
                    metrich[a, -e] metrich[c, -f] n[b] n[
                    d] WeylCD[-a, -b, -c, -d] // WeylToRiemann, 
                    metrich] // Proj // 
                    ContractMetric // NoScalar // Simplification, CD, 
                    cd] // GradNormalToExtrinsicK // ContractMetric //
                     Simplification) //. TraceFreeKRule // 
                 ContractMetric // Simplification) //. 
              TraceFreeKRule // ContractMetric) /. 
           TraceLienAARule //. TraceFreeKRule // Simplification) //. 
       TraceFreeRule //. AccRule // ContractMetric // Simplification //
	  Expand];

Bdefn[-e_, -f_] = 
 Module[{a, b, c, 
   d}, (ToInducedDerivative[
          GaussCodazzi[
               metrich[a, -e] metrich[c, -f] n[b] n[
                   d] (WeylStar[-a, -b, -c, -d] + 
                    WeylStar[-c, -b, -a, -d])/2 // DecomposeEps // 
                WeylToRiemann, metrich] // Proj // 
             ContractMetric // NoScalar // Simplification, CD, cd] // 
         GradNormalToExtrinsicK // ContractMetric // Simplification //
       ContractMetric) /. TraceFreeKRule // Simplification];

BWeylToDefnRule=MakeRule[{BWeyl[-a,-b],Evaluate@Bdefn[-a,-b]}];

(***********************************************************************)
(**********               E  from GR                         **********)
(***********************************************************************)

DefTensor[EWeylGR[-a,-b],M,Symmetric[{1,2}],OrthogonalTo->n[a], 
ProjectedWith->{metrich[a,-c],metrich[b,-d]},PrintAs->"EGR"];

EWeylGR /: EWeylGR[-a_, a_] := 0
EWeylGR /: EWeylGR[a_, -a_] := 0

EGRdefn[-a_,-b_]= Module[{c,d,k},Riccicd[-a,-b]
				-metrich[-a,-b]metrich[c,d]Riccicd[-c,-d]/3
				+TrK[]AA[-a,-b]/3-AA[k,-a]AA[-k,-b]
				+AA[c,d]AA[-c,-d]metrich[-a,-b]/3];

EToXRule=MakeRule[{EWeyl[-a,-b],EWeylGR[-a,-b]+X[-a,-b]},MetricOn->All];

(* Rule to replace Rtf with the electric part from GR *)
RtToEWeylGRRule=MakeRule[{Rt[-a,-b], EWeylGR[-a,-b]+AA[-a,c]AA[-b,-c] 
			-1/3 metrich[-a,-b]AA[c,d]AA[-c,-d]-1/3 TrK[]AA[-a,-b]}];

RtToEWeylGR = Function[X, X/.RtToEWeylGRRule];

(***********************************************************************)
(**********               Constraints from GR                **********)
(***********************************************************************)

(* Place holder for Hamiltonian constraint *)
DefTensor[Ham[], M, PrintAs -> "H"];

(* Place holder for Hamiltonian constraint *)
DefTensor[HGR[], M, PrintAs -> "HGR"];

(* Replace R with H *)
RicciToHGRRule=MakeRule[{RicciScalarcd[] ,HGR[]+(AA[-a, -b]*AA[a, b])-(2*TrK[]^2)/3}];
RicciToHGR=Function[X,X/.RicciToHGRRule];

(* Place holder for GR momentum constraint *)
DefTensor[MGR[-a],M,OrthogonalTo->n[a],
			ProjectedWith->{metrich[a,-c]},PrintAs->"MGR"];

(* Replace Grad K with M *)
GradKToMGRRule=MakeRule[{cd[-a][TrK[]],3/2cd[-b][AA[-a,b]]-3/2MGR[-a]}];
GradKToMGR=Function[X,X/.GradKToMGRRule];

(***********************************************************************)
(**********        Chern-Simons fields and constants          **********)
(***********************************************************************)
DefTensor[theta[], M, PrintAs -> "\[Theta]"];
DefTensor[PI[], M, PrintAs -> "\[CapitalPi]"];

DefConstantSymbol[A]; DefConstantSymbol[B]; DefConstantSymbol[Bo2,PrintAs->"\!\(\*FractionBox[\(B\), \(2\)]\)"];

(* Notice the sign in the defn here *)
ThetaDotRule = MakeRule[{LieD[n[a]][theta[]], PI[]}];

PiDotRule = 
  MakeRule[{LieD[n[a]][PI[]], 
    TrK[] * PI[] - (4*A*BWeyl[a, b]*EWeyl[-a, -b])/
      Scalar[B] + Accelerationn[a]*cd[-a][theta[]] + 
     metrich[a, b]*cd[-b][cd[-a][theta[]]]}];

(***********************************************************************)
(**********            Decomposition of the Ricci             **********)
(***********************************************************************)
(* double contraction *)
Rnn = Module[{a, 
   b}, (ToInducedDerivative[
          GaussCodazzi[n[a] n[b] RicciCD[-a, -b], metrich] // 
            NoScalar // ContractMetric, CD, cd] /. TraceFreeKRule // 
        ContractMetric // Simplification) /. TraceFreeRule /. 
    AccRule // ContractMetric];

(* Mixed - Contraction - Projection *)
RnProj[-c_] = 
 Module[{a, b}, 
  ToInducedDerivative[
    GaussCodazzi[metrich[a, -c] n[b] RicciCD[-a, -b], metrich] // 
      NoScalar // ContractMetric, CD, cd] // ContractMetric];

(* Double projection *)
RnDProj[-c_, -d_] = Module[{a, 
   b},GradNormalToExtrinsicK[((ToInducedDerivative[
           GaussCodazzi[metrich[a, -c] metrich[b, -d] RicciCD[-a, -b],
               metrich] // NoScalar // ContractMetric, CD, cd] // 
          GradNormalToExtrinsicK // ContractMetric // Proj ) /. TraceLienRules /. 
     TraceFreeRule/.AccRule/. 
     TraceFreeRule//Expand//GradNormalToExtrinsicK)/.AccRule
//GradNormalToExtrinsicK,metrich]//Expand];

(* Trace of Double projection *)
RProjTr = 
 Module[{a, 
   b}, (RnDProj[-a, -b] metrich[a, b] /. TraceFreeKRule // 
      ContractMetric) /. TraceLienRules // Simplification];

(***********************************************************************)
(**********   Geometry of the Electric and magnetic parts   **********)
(***********************************************************************)
(* This rule is derived in the notebook *)

DivEGRRule = 
MakeRule[{cd[-b][EWeylGR[-a,b]],
-1/2AA[-a,-b]MGR[b]+MGR[-a]TrK[]/3+1/6cd[-a]@HGR[]+epsilonmetrich[-a,b,c]AA[-b,d]BWeyl[-c,-d]},
MetricOn->All];

(* 
(* Make a function that only has A dot, to be sure that nothing else is used. *)
ADot=Function[X,X/.ADotRule];

(* Make a rule for  *)
DivAToMGRRule=MakeRule[{cd[-c][AA[-a,c]],MGR[-a]+2/3 cd[-a][TrK[]]}];

(* Compute! *)
(ToInducedDerivative[3/2(n[b]epsilonmetrich[-h,d,c](metric[e,a]Antisymmetrize[CD[-e]@(RiemannCD[-a,-b,-c,-d])
,{-e,-c,-d}]//RiemannToWeyl)//WeylToPart//Simplification//GaussCodazzi//NoScalar)/.Projectormetrich->ProjectWith[metrich]
//ContractMetric,CD,cd]//ContractMetric//GradNormalToExtrinsicK//TraceFree//ADot//Simplification//TraceFree//SplitMetric
//RtToEWeylGR//ContractMetric//Simplification)/.DivAToMGRRule//Simplification;
Clear[ADot,DivAToMGRRule];
*)

DivBRule = MakeRule[{cd[-a][BWeyl[-h, a]],-AA[a, b]*epsilonmetrich[-h, -b, -c]*EWeylGR[-a, c]  - 
 (epsilonmetrich[-h, a, b]*cd[-b][MGR[-a]])/2}];

Simp[e_] := 
 e // GradNormalToExtrinsicK // ContractMetric // Simplification
TraceFree[e_] := e /. TraceFreeRule



(***********************************************************************)
(**********            Decomposition of C-Tensor              **********)
(***********************************************************************)

CC[-a_, -b_] = 
  Module[{c, d}, 
   Symmetrize[
     2 (CD[c]@theta[]) CD[d]@
        WeylStar[-d, -a, -b, -c] + (CD[c]@
         CD[d]@theta[]) WeylStar[-a, -c, -d, -b], {-a, -b}] // 
    Simplification];

C2[-a_, -b_] = 
  Module[{c, d}, 
   Symmetrize[ (CD[c]@
         CD[d]@theta[]) WeylStar[-a, -c, -d, -b], {-a, -b}] // 
    Simplification];

    
(* double contraction *)
Cnn = Module[{a, b}, 
  ToInducedDerivative[(ToInducedDerivative[
          n[a] n[b] CC[-a, -b] //WeylToPart //DecomposeEps // 
              ContractMetric // Simplification // 
            GradNormalToExtrinsicK // Simplification, CD, cd]) /. 
        ThetaDotRule // ContractMetric, CD, cd] // 
     GradNormalToExtrinsicK // ContractMetric 
     // Simplification]/.DivBRule//Simplification;

C2nn = Module[{a, b}, 
  ToInducedDerivative[(ToInducedDerivative[
          n[a] n[b] C2[-a, -b] //WeylToPart //DecomposeEps // 
              ContractMetric // Simplification // 
            GradNormalToExtrinsicK // Simplification, CD, cd]) /. 
        ThetaDotRule // ContractMetric, CD, cd] // 
     GradNormalToExtrinsicK // ContractMetric 
     // Simplification]/.DivBRule//Simplification;

(* Mixed - Contraction - Projection *)
CnProj[-c_] = 
  Module[{a, b}, 
     ToInducedDerivative[
            ToInducedDerivative[
                  metrich[a, -c] n[b] CC[-a, -b] // WeylToPart // 
                    DecomposeEps // ContractMetric // Simplification // 
                    GradNormalToExtrinsicK // Simplification, CD, 
                  cd] /. ThetaDotRule // ContractMetric // 
               GradNormalToExtrinsicK // ContractMetric // 
             Simplification, CD, cd] /. ThetaDotRule // 
          ContractMetric // GradNormalToExtrinsicK // ContractMetric //
        Simplification // ContractMetric] /. TraceFreeKRule /.DivBRule // 
   Simplification;

C2nProj[-c_] = 
  Module[{a, b}, 
     ToInducedDerivative[
            ToInducedDerivative[
                  metrich[a, -c] n[b] C2[-a, -b] // WeylToPart // 
                    DecomposeEps // ContractMetric // Simplification // 
                    GradNormalToExtrinsicK // Simplification, CD, 
                  cd] /. ThetaDotRule // ContractMetric // 
               GradNormalToExtrinsicK // ContractMetric // 
             Simplification, CD, cd] /. ThetaDotRule // 
          ContractMetric // GradNormalToExtrinsicK // ContractMetric //
        Simplification // ContractMetric] /. TraceFreeKRule /.DivBRule // 
   Simplification;

   
(* Double projection *)
CProj[-c_, -d_] = (Module[{a, 
         b}, (ToInducedDerivative[
                 ToInducedDerivative[
                    metrich[a, -c] metrich[b, -d] CC[-a, -b] // 
                    WeylToPart // DecomposeEps // ContractMetric // 
                    Simplification // GradNormalToExtrinsicK // 
                    Simplification, CD, cd] /. ThetaDotRule // 
                    ContractMetric // GradNormalToExtrinsicK // 
                   ContractMetric // Simplification, CD, cd] // 
                GradNormalToExtrinsicK // ContractMetric // 
              Simplification) /. PiDotRule // ContractMetric // 
          Simplification // ContractMetric] /. TraceFreeKRule /. 
      TraceLienRules /. AccRule /.DivBRule // ContractMetric // Simplification);

C2Proj[-c_, -d_] = (Module[{a, 
         b}, (ToInducedDerivative[
                 ToInducedDerivative[
                    metrich[a, -c] metrich[b, -d] C2[-a, -b] // 
                    WeylToPart // DecomposeEps // ContractMetric // 
                    Simplification // GradNormalToExtrinsicK // 
                    Simplification, CD, cd] /. ThetaDotRule // 
                    ContractMetric // GradNormalToExtrinsicK // 
                   ContractMetric // Simplification, CD, cd] // 
                GradNormalToExtrinsicK // ContractMetric // 
              Simplification) /. PiDotRule // ContractMetric // 
          Simplification // ContractMetric] /. TraceFreeKRule /. 
      TraceLienRules /. AccRule /.DivBRule // ContractMetric // Simplification);

      
(* Trace of Double projection *)
CProjTr = (((ToInducedDerivative[
                 ToInducedDerivative[
                    metrich[a, d] metrich[b, -d] CC[-a, -b] // 
                    WeylToPart // DecomposeEps // ContractMetric // 
                    Simplification // GradNormalToExtrinsicK // 
                    Simplification, CD, cd] /. ThetaDotRule // 
                    ContractMetric // GradNormalToExtrinsicK // 
                   ContractMetric // Simplification, CD, cd] // 
                GradNormalToExtrinsicK // ContractMetric // 
              Simplification) /. PiDotRule // 
          ContractMetric // Simplification // ContractMetric) /. 
      TraceLienBWeylRule //. TraceFreeKRule //. DivBRule // ContractMetric // 
	     Simplification);

	     
C2ProjTr = (((ToInducedDerivative[
                 ToInducedDerivative[
                    metrich[a, d] metrich[b, -d] C2[-a, -b] // 
                    WeylToPart // DecomposeEps // ContractMetric // 
                    Simplification // GradNormalToExtrinsicK // 
                    Simplification, CD, cd] /. ThetaDotRule // 
                    ContractMetric // GradNormalToExtrinsicK // 
                   ContractMetric // Simplification, CD, cd] // 
                GradNormalToExtrinsicK // ContractMetric // 
              Simplification) /. PiDotRule // 
          ContractMetric // Simplification // ContractMetric) /. 
      TraceLienBWeylRule //. TraceFreeKRule //. DivBRule // ContractMetric // 
	     Simplification);

(***********************************************************************)
(**********               Dot rule so far                     **********)
(***********************************************************************)
DotRule = {ThetaDotRule, PiDotRule}//Flatten;
Dots=Function[X,X/.DotRule];

(***********************************************************************)
(**********               Energy momentum tensor              **********)
(***********************************************************************)
DefTensor[TT[-a, -b], M, Symmetric[{-a, -b}]];

(* Notice that this is the trace-reversed Energy momentum tensor! *)
	    
	    TT[-a_, -b_] := CD[-a][theta[]] CD[-b][theta[]];
tt[-a_, -b_] := 
 Module[{c, d}, 
   metrich[-a, c] metrich[-b, d] ToInducedDerivative[TT[-c, -d], CD, 
     cd]] // Expand;
Trtt = tt[-a, -b] metrich[a, b] // ContractMetric;
tn[-a_] = 
 Module[{b, c}, 
    metrich[-a, b] n[c] ToInducedDerivative[TT[-b, -c], CD, cd]] /. 
   DotRule // Expand;
tnn = Module[{a, b}, 
   n[a] n[b] ToInducedDerivative[TT[-a, -b], CD, cd] // Expand] /. DotRule;


(***********************************************************************)
(**********               Rule for TrK dot                   **********)
(***********************************************************************)

(* Following rule comes from: *)
(* RProjTr + A CProjTr - Bo2 Trtt//TraceFree//Simplification//ContractMetric *)

TrKDotRule= MakeRule[{LieD[n[a]][TrK[]],
AA[-a, -b]*AA[a, b] + HGR[] - A*AA[a, b]*BWeyl[-a, -b]*PI[] + TrK[]^2/3 - 
 Bo2*metrich[a, b]*cd[-a][theta[]]*cd[-b][theta[]] - 
 (metrich[a, b]*cd[-b][cd[-a][lapse[]]])/lapse[] - 
 A*BWeyl[a, b]*cd[-b][cd[-a][theta[]]] + A*epsilonmetrich[a, c, b]*cd[-b][theta[]]*
  cd[-c][MGR[-a]] - 2*A*AA[a, c]*epsilonmetrich[-c, b, d]*X[-a, -b]*cd[-d][theta[]]}];

TrKDotRule2=MakeRule[{LieD[n[a]][TrK[]],-(A*AA[a, b]*BWeyl[-a, -b]*PI[]) + RicciScalarcd[] + TrK[]^2 - 
 Bo2*metrich[a, b]*cd[-a][theta[]]*cd[-b][theta[]] - (metrich[a, b]*cd[-b][cd[-a][lapse[]]])/lapse[] - 
 A*BWeyl[a, b]*cd[-b][cd[-a][theta[]]]}];
  
DotRule=Union[DotRule,TrKDotRule];
DotRule2=Union[DotRule,TrKDotRule2];
Dots=Function[X,X/.DotRule];
Dots2=Function[X,X/.DotRule2];

(***********************************************************************)
(**********               Rule for A dot                     **********)
(***********************************************************************)

(* Great care is needed in applying this rule! *)

ADotRule=MakeRule[{LieD[n[a]][AA[-e,-f]],
-(Accelerationn[-e]*Accelerationn[-f]) + 2*EWeyl[-e, -f] - 
 (2*AA[-a, -b]*AA[a, b]*metrich[-e, -f])/3 + 
 (Accelerationn[-a]*Accelerationn[a]*metrich[-e, -f])/3 - Rt[-e, -f] - 
 (AA[-e, -f]*TrK[])/3 + (metrich[-e, -f]*cd[-a][Accelerationn[a]])/3 - 
 cd[-f][Accelerationn[-e]]}];

DotRule=Union[DotRule,ADotRule];
Dots=Function[X,X/.DotRule];

(***********************************************************************)
(**********      Rule for the Spatial Curl of Ricci         **********)
(***********************************************************************)

(* The following few lines derive BDot *)

(*

(* Start with the Bianchi identity *)
CurlRicciRule=MakeRule[{epsilonmetrich[e,d,-g]metrich[f,-c]CD[-d][RicciCD[-e,-f]],
Evaluate@((((ToInducedDerivative[-epsilonmetrich[-g,a,b]metrich[d,-a]metrich[e,-b]metrich[f,-c]
ToInducedDerivative[CD[-d][(GaussCodazzi[RicciCD[-e,-f],metrich]//NoScalar
//AccToLapse//Proj)//NoScalar//Simplification],CD,cd]//GradNormalToExtrinsicK
//ContractMetric//AccToLapse//Simplification//ContractMetric//Simplification,CD,cd]
//ContractMetric//Simplification)//TraceFree//ContractMetric)//Dots//TraceFree
//AccToLapse//ContractMetric//Simplification//ContractMetric)//RtToEWeylGR
//Simplification//ContractMetric//Simplification//ContractMetric)}];

CurlRicci = Function[X, X/.CurlRicciRule];

hnBianchitmp[-g_,-f_]=Module[{d,e,a,b,c},
metrich[d,-g]n[e](epsilonmetric[a,b,c,-f]CD[-a][RiemannCD[-b,-c,-d,-e]]
//RiemannToWeyl//WeylToPart//DecomposeEps)//Simp];

hnBianchi[-a_,-b_]=ToInducedDerivative[
hnBianchitmp[-a,-b]/4+hnBianchitmp[-b,-a]/4,CD,cd]//Simp//ProjectWith[metrich]//Simplification//TraceFree
//ContractMetric//Simplification//ContractMetric;Clear[hnBianchitmp];
-hnBianchi[-a,-b]//CurlRicci//AccToLapse//GradKToMGR//ContractMetric//Simplification//ContractMetric
//ScreenDollarIndices//InputForm;
Clear[CurlRicci, CurlRicciRule];
*)

BDotRule=MakeRule[{ metrich[-a, c]*metrich[-b, d]*LieD[n[a]][BWeyl[-c, -d]],
(-5*AA[-b, c]*BWeyl[-a, -c])/2 - (5*AA[-a, c]*BWeyl[-b, -c])/2 + 
 AA[c, d]*BWeyl[-c, -d]*metrich[-a, -b] + (AA[-b, c]*epsilonmetrich[-a, -c, d]*MGR[-d])/
  4 + (AA[-a, c]*epsilonmetrich[-b, -c, d]*MGR[-d])/4 + (BWeyl[-a, -b]*TrK[])/3 - 
 epsilonmetrich[-b, c, d]*cd[-d][EWeyl[-a, -c]] - 
 epsilonmetrich[-a, c, d]*cd[-d][EWeyl[-b, -c]] + 
 (epsilonmetrich[-b, c, d]*cd[-d][EWeylGR[-a, -c]])/2 + 
 (epsilonmetrich[-a, c, d]*cd[-d][EWeylGR[-b, -c]])/2 - 
 (epsilonmetrich[d, -b, -c]*EWeyl[-a, c]*cd[-d][lapse[]])/lapse[] - 
 (epsilonmetrich[d, -a, -c]*EWeyl[-b, c]*cd[-d][lapse[]])/lapse[]},MetricOn->{a,b}];

(* Defining Cheat rule for when we don't have the projection stood around. Use with care! *)
BDotRuleCheat=MakeRule[{ LieD[n[a]][BWeyl[-a, -b]],
(-5*AA[-b, c]*BWeyl[-a, -c])/2 - (5*AA[-a, c]*BWeyl[-b, -c])/2 + 
 AA[c, d]*BWeyl[-c, -d]*metrich[-a, -b] + (AA[-b, c]*epsilonmetrich[-a, -c, d]*MGR[-d])/
  4 + (AA[-a, c]*epsilonmetrich[-b, -c, d]*MGR[-d])/4 + (BWeyl[-a, -b]*TrK[])/3 - 
 epsilonmetrich[-b, c, d]*cd[-d][EWeyl[-a, -c]] - 
 epsilonmetrich[-a, c, d]*cd[-d][EWeyl[-b, -c]] + 
 (epsilonmetrich[-b, c, d]*cd[-d][EWeylGR[-a, -c]])/2 + 
 (epsilonmetrich[-a, c, d]*cd[-d][EWeylGR[-b, -c]])/2 - 
 (epsilonmetrich[d, -b, -c]*EWeyl[-a, c]*cd[-d][lapse[]])/lapse[] - 
 (epsilonmetrich[d, -a, -c]*EWeyl[-b, c]*cd[-d][lapse[]])/lapse[]}];

DotRule=Union[DotRule,BDotRule];
Dots=Function[X,X/.DotRule];


(* Define L - scalar for the magnitude of D_i theta. *)
DefTensor[L[], M];

(* Define Xss *)
DefTensor[Xss[], M];

(* Define the unit vector s and the associated projection operator *)
DefTensor[s[a], M,
 OrthogonalTo -> {n[-a]}, 
	  ProjectedWith -> {metrich[-a, c]}];

s /: s[a_] s[-a_] := 1;
s /: s[a_] s[b_] metric[-a_, -b_] := 1;
s /: s[-a_] s[-b_] metric[a_, b_] := 1;
s /: s[a_] cd[b_][s[-a_]] := 0;
s /: s[a_] cd[-b_][s[-a_]] := 0;
s /: s[-a_] cd[b_][s[a_]] := 0;
s /: s[-a_] cd[-b_][s[a_]] := 0;
s /: epsilonmetrich[-a_, c_, b_]*s[a_]*cd[-b_][s[-c_]] := 0;
s /: epsilonmetrich[-a_, -c_, -b_]*s[a_]*cd[b_][s[c_]] := 0;

(* Define the O operator. Note that this is not properly defined as a tensor. *)
IndexSet[Oop[-a_,-b_,c_,d_],1/2(epsilonmetrich[-a,c,e]metrich[d,-b]+epsilonmetrich[-b,c,e]metrich[d,-a])cd[-e][theta[]]]

(* Define X tilde and X tilde dot *)
DefTensor[Xtilde[-a,-b],M,Symmetric[{1,2}],OrthogonalTo->n[a], 
ProjectedWith->{metrich[a,-c],metrich[b,-d]},PrintAs->"Xt"];
DefTensor[DotXtilde[-a,-b],M,Symmetric[{1,2}],OrthogonalTo->n[a], 
ProjectedWith->{metrich[a,-c],metrich[b,-d]},PrintAs->"DotXt"];
DefTensor[DotDotXtilde[-a,-b],M,Symmetric[{1,2}],OrthogonalTo->n[a], 
ProjectedWith->{metrich[a,-c],metrich[b,-d]},PrintAs->"DotDotXt"];

(* Xtilde is Tracefree *)
Xtilde /: Xtilde[-a_, a_] := 0
Xtilde /: Xtilde[a_, -a_] := 0
Xtilde /: metrich[a_,b_] Xtilde[-a_, -b_] := 0
Xtilde /: metric[a_,b_] Xtilde[-a_, -b_] := 0
(* Xtilde is orthogonal to ss *)
Xtilde /: s[a_]s[b_]Xtilde[-a_, -b_] := 0
Xtilde /: s[-a_]s[-b_]Xtilde[a_, b_] := 0
Xtilde /: s[a_]s[-b_]Xtilde[-a_, b_] := 0
Xtilde /: s[-a_]s[b_]Xtilde[a_, -b_] := 0

(* Ass *)
DefTensor[Ass[],M];
AssRule=MakeRule[{s[a]s[b]AA[-a,-b],Ass[]},MetricOn->All];

(* Bss *)
DefTensor[Bss[], M];
BssRule=MakeRule[{s[a]s[b]BWeyl[-a,-b],Bss[]},MetricOn->All];


(* Derive Rule on Grad Dot theta *)
(*
ToInducedDerivative[ToInducedDerivative[LieDToCovD[LieD[n[-a]][cd[-a][theta[]]],CD]
/.cd[-a_][theta[]]->metrich[-a,b]CD[-b][theta[]]//Simplification,CD,cd]
//GradNormalToExtrinsicK//Dots//AccToLapse//ContractMetric//Simplification,CD,cd]
//ContractMetric//GradNormalToExtrinsicK//TraceFree//ContractMetric//SplitMetric
//TraceFree//Simplification//ContractMetric//ScreenDollarIndices//InputForm;
*)

DotGradThetaRule=MakeRule[{LieD[n[-a]][cd[-a][theta[]]],(PI[]*cd[-a][lapse[]])/lapse[]
+cd[-a][PI[]]},MetricOn->All];

GradThetaSquaredToLRules={
MakeRule[{metrich[a,b]cd[-a][theta[]]cd[-b][theta[]],L[]^2},MetricOn->All],
MakeRule[{cd[a][theta[]]cd[-a][theta[]],L[]^2},MetricOn->All]}//Flatten;

GradThetaTosRule = MakeRule[{cd[-a][theta[]],L[]s[-a]},MetricOn->All];
GradsToLRule=MakeRule[{s[b]cd[-b][s[-a]],(cd[-a][L[]]-s[-a]*s[b]*cd[-b][L[]])/L[]},MetricOn->All];

(* ((s[b]cd[-b][s[-a]]/.MakeRule[{s[-a],cd[-a][theta[]]/L[]},MetricOn->All]//ContractMetric)//ContractMetric)
/.MakeRule[{cd[b][theta[]]cd[-b][cd[-a][theta[]]],L[]cd[-a][L[]]},MetricOn->All]
/.MakeRule[{metrich[b,c]cd[-c][theta[]]cd[-b][cd[-a][theta[]]],L[]cd[-a][L[]]},MetricOn->All]
/.GradThetaTosRule//Simplification//InputForm *)

(* (s[-a]s[-b]-1/2(metrich[-a,-b]-s[-a]s[-b]))Xss[]+Symmetrize[4L[]^-1 Xtilde[-d,-e]s[e]
epsilonmetrich[c,d,-a]s[-c]s[-b],{-a,-b}]+Symmetrize[L[]^-1Xtilde[-c,-d]
epsilonmetrich[e,c,-a]s[-e](metrich[d,-b]-s[d]s[-b]),{-a,-b}]
//ContractMetric//Simplification//InputForm *)

(* replace X with Xtilde and Xss *)
XtoXTildeRule=MakeRule[{X[-a,-b],(-(L[]*metrich[-a,-b]*Xss[])+s[c]*
(epsilonmetrich[-b,-c,-d]*Xtilde[-a,d]+epsilonmetrich[-a,-c,-d]*Xtilde[-b,d]
+3*epsilonmetrich[-a,-d,-e]*s[-b]*s[d]*Xtilde[-c,e])+3*s[-a]
*(L[]*s[-b]*Xss[]+epsilonmetrich[-b,-d,-e]*s[c]*s[d]*Xtilde[-c,e]))/(2*L[])},MetricOn->All];

XtoXTilde = Function[X,X/.XtoXTildeRule];

(* Define Scalar function for the coefficient in the Xss equation *)
DefTensor[XssCoeff[], M,PrintAs->(*"ooBm3A2Bss2"*) "CoX"];

(* Convenient Function for seperate metric *)
SplitMetric=Function[X,SeparateMetric[metric][X]];
ToInducedMetricRule=MakeRule[{metric[-a,-b],metrich[-a,-b]-n[-a]n[-b]}];

(* A couple of generic functions for killing Rs and commuting cd's *)
(* Probably not used *)
DropR=(#/.Riemanncd[a_,b_,c_,d_]->0/.Riccicd[a_,b_]->0/.RicciScalarcd[]->0)&;
SwapDrop=(CommuteCovDs[#1,cd,#2]//DropR//ContractMetric//Simplification//ContractMetric)&;
